# Generated by roxygen2: do not edit by hand

S3method("$",ffnn_fit)
S3method("$",layer_pr)
S3method("$",nn_fit)
S3method("$",rnn_fit)
S3method(autoplot,nn_fit)
S3method(garson,ffnn_fit)
S3method(grid_depth,default)
S3method(grid_depth,list)
S3method(grid_depth,model_spec)
S3method(grid_depth,param)
S3method(grid_depth,parameters)
S3method(grid_depth,workflow)
S3method(olden,ffnn_fit)
S3method(plot,nn_fit)
S3method(predict,ffnn_fit)
S3method(predict,nn_fit)
S3method(predict,nn_fit_ds)
S3method(predict,nn_fit_tab)
S3method(predict,rnn_fit)
S3method(print,early_stop_spec)
S3method(print,ffnn_fit)
S3method(print,layer_index_pr)
S3method(print,layer_input_pr)
S3method(print,layer_is_output_pr)
S3method(print,layer_output_pr)
S3method(print,layer_pr)
S3method(print,mlp_kindling)
S3method(print,nn_arch)
S3method(print,nn_fit)
S3method(print,rnn_fit)
S3method(print,rnn_kindling)
S3method(print,train_nnsnip)
S3method(train_nn,data.frame)
S3method(train_nn,dataset)
S3method(train_nn,default)
S3method(train_nn,formula)
S3method(train_nn,matrix)
S3method(translate,mlp_kindling)
S3method(translate,rnn_kindling)
S3method(translate,train_nnsnip)
S3method(tunable,mlp_kindling)
S3method(tunable,rnn_kindling)
S3method(tunable,train_nnsnip)
S3method(update,mlp_kindling)
S3method(update,rnn_kindling)
S3method(update,train_nnsnip)
export(.i)
export(.in)
export(.is_output)
export(.layer)
export(.out)
export(act_funs)
export(activations)
export(args)
export(autoplot_diagnostics)
export(bias)
export(bidirectional)
export(early_stop)
export(ffnn)
export(ffnn_generator)
export(ffnn_wrapper)
export(garson)
export(grid_depth)
export(hidden_neurons)
export(mlp_kindling)
export(n_hlayers)
export(new_act_fn)
export(nn_arch)
export(nn_module_generator)
export(olden)
export(optimizer)
export(output_activation)
export(plot_diagnostics)
export(rnn)
export(rnn_generator)
export(rnn_kindling)
export(rnn_wrapper)
export(table_summary)
export(train_nn)
export(train_nn_wrapper)
export(train_nnsnip)
export(validation_split)
importFrom(NeuralNetTools,garson)
importFrom(NeuralNetTools,olden)
importFrom(cli,cat_line)
importFrom(cli,cli_abort)
importFrom(cli,rule)
importFrom(cli,style_bold)
importFrom(cli,style_italic)
importFrom(ggplot2,autoplot)
importFrom(glue,glue)
importFrom(lifecycle,badge)
importFrom(parsnip,translate)
importFrom(purrr,imap)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,set_names)
importFrom(purrr,transpose)
importFrom(rlang,":=")
importFrom(rlang,as_string)
importFrom(rlang,call2)
importFrom(rlang,call_name)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(rlang,eval_tidy)
importFrom(rlang,exec)
importFrom(rlang,expr)
importFrom(rlang,f_rhs)
importFrom(rlang,is_call)
importFrom(rlang,is_formula)
importFrom(rlang,is_installed)
importFrom(rlang,new_function)
importFrom(rlang,quo_get_expr)
importFrom(rlang,quo_is_null)
importFrom(rlang,sym)
importFrom(stats,reorder)
importFrom(stats,setNames)
importFrom(stats,update)
importFrom(tune,tunable)
importFrom(vctrs,new_vctr)
