% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/early_stop.R
\name{early_stop}
\alias{early_stop}
\title{Early Stopping Specification}
\usage{
early_stop(
  patience = 5L,
  min_delta = 1e-04,
  restore_best_weights = TRUE,
  monitor = "val_loss"
)
}
\arguments{
\item{patience}{Integer. Epochs to wait after last improvement. Default \code{5}.}

\item{min_delta}{Numeric. Minimum improvement to qualify as better. Default \code{1e-4}.}

\item{restore_best_weights}{Logical. Restore weights from best epoch. Default \code{TRUE}.}

\item{monitor}{Character. Metric to monitor. One of \code{"val_loss"} (default) or \code{"train_loss"}.}
}
\value{
An object of class \code{"early_stop_spec"}.
}
\description{
\code{early_stop()} is a helper function to be supplied on \code{early_stopping} arguments.
}
