% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/torch-code-gen.R
\name{formula_to_function}
\alias{formula_to_function}
\title{Formula to Function with Named Arguments}
\usage{
formula_to_function(
  formula_or_fn,
  default_fn = NULL,
  arg_names = NULL,
  alias_map = NULL
)
}
\arguments{
\item{formula_or_fn}{A formula or function}

\item{default_fn}{Default function if \code{formula_or_fn} is \code{NULL}}

\item{arg_names}{Character vector of formal argument names}

\item{alias_map}{Named list mapping arg_names to formula aliases (e.g., list(in_dim = ".in"))}
}
\value{
A function
}
\description{
Formula to Function with Named Arguments
}
\keyword{internal}
